"""
Utilities for plotting and scanning the bandstructure
"""
import numpy as np
from numpy.linalg import norm
import itertools

def route_through_zone(points, Npts=40):
    p0 = points[0]
    segments = []
    seglengths = []
    seglen = 0.0
    segboundaries = [0]
    for p in points[1:]:
        seg = []
        for r in np.linspace(0.0, 1.0, Npts)[:-1]:
            seg.append( p0 + r*(p-p0) )
            seglen += 1.0/float(Npts)# * norm(p-p0)
            seglengths.append(seglen)
        segments += seg
        segboundaries.append(seglen)
        p0 = p
#    from matplotlib import pyplot as plt
#    plt.xlim((-1,1))
#    plt.ylim((-1,1))
    print "Symmetry Points"
    for p in points:
        print p
    print "Segments"
    for p in segments:
        print p
#        plt.plot(p[0], p[1], "o")
#    plt.show()
    return seglengths, segboundaries, segments

def plot_1d_bandstructure(points, point_names, seglengths, segboundaries, band_structure, highlight_bands=[]):
    from matplotlib import pyplot as plt
    plt.xlabel("k")
    plt.ylabel("energy / eV")
    bands = np.vstack(band_structure)
    nk, nb = bands.shape
    y = np.linspace(bands.min(), bands.max(), 100)*27.211
    for ib in range(0, nb):
        if ib in highlight_bands:
            plt.plot(seglengths, bands[:,ib] * 27.211, lw=2)
        else:
            plt.plot(seglengths, bands[:,ib] * 27.211, "-.", lw=1)
    for segb in segboundaries:
        plt.plot(segb*np.ones(y.shape), y, ls="-", color="black", lw=0.5)
    ax = plt.gca()
    ax.set_xticks(segboundaries)
    ax.set_xticklabels(point_names)
    plt.draw()
    plt.savefig("/tmp/bandstructure.png")
    plt.draw()
    plt.show()
             

def write_2d_bandstructure(filename, ks, band_structure):
    fh = open(filename, "w")
    print "len(ks) = %s" % len(ks)
    print "len(band_structure) = %s" % len(band_structure)
    for ((kx,ky,kz), orbe) in itertools.izip(ks, band_structure):
        print>>fh, "%.5f %.5f %.5f    " % (kx,ky,kz),
#        print>>fh, "%.5f %.5f    " % (kx,ky),
#        print>>fh, "%.5f    " % (kx),
        for en in orbe:
            print>>fh, "%s " % en,
        print>>fh, ""
    fh.close()
