import numpy as np

from DFTB import Bandstructure
from DFTB import Periodic, XYZ
from DFTB.DFTB2 import DFTB2

def carbon_fcc(a0):
    lat = Periodic.FaceCenteredCubic(a0)
    unitcell = [(6,(np.array([0.0,0.0,0.0])))]
    P = lat.symmetry_kpoints()

    dftb2 = DFTB2(unitcell, parameter_set="homegrown", verbose=1)
    dftb2.setGeometry(unitcell)

    route = [P["W"], P["X"], P["Gamma"], P["L"], P["K"], P["Gamma"], P["L"]]
    names = ["W", "X", "$\Gamma$", "L", "K", "$\Gamma$", "L"]
    seglengths, segboundaries, kpoints = Bandstructure.route_through_zone(route)

    lattice_vectors = lat.getLatticeVectors(space="real")
    lattice, band_structure, dq = dftb2.runPeriodicSCC(lattice_vectors, kpoints, nmax=3)

    XYZ.write_xyz("carbon_fcc_lattice.xyz", [lattice])

    Bandstructure.plot_1d_bandstructure(route, names, seglengths, segboundaries, band_structure)
    return dftb2.E_tot

def plot_a0_curve():
    lattice_constants = np.linspace(4.0, 10.0, 100)
    total_energies = []
    for a0 in lattice_constants:
        total_energies.append( carbon_fcc(a0) )
    total_energies = np.array(total_energies)

    from matplotlib import pyplot as plt
    plt.xlabel("lattice constant a0 / bohr")
    plt.ylabel("total energy / eV")
    plt.plot(lattice_constants, total_energies*27.211)
    plt.draw()
    plt.show()


if __name__ == "__main__":
    carbon_fcc(6.0)
